% This example file computes the equilibrium for rI=50, sigma=0.2,
% and tariff levels tau=0,20,30,40

% The resulting price-cost margin corresponds to Figure 13 of the
% paper

% Similarly, the comsumer gain, producer loss, and welfare loss are
% computed (corresponds to selected points in Figure 14 of the paper)


% Investment cost:
breakevenprice=50; % Investment cost is here written in units comparable with prices: this price, if it would prevail forever, 
% would make the entrant just indefferent between entry and no-entry
Iflow=breakevenprice*8760; %investment cost in flow units (annual rental cost of capital; in continuous time, Iflow=rI)

% parameters of oil price process in continuous time:
mu=0;
sigma=0.2;

%parameters for the corresponding discrete time binomial process:
delta=0.02; %period length in years
alfa=exp(sigma*sqrt(delta)); % multiplier to get next period value
q=(1/2)*(1+(mu-sigma^2/2)*sqrt(delta)/sigma); % probability of going up

% Discounting parameters:
r=0.05; % Discount rate
disc=exp(-r*delta); % discount factor

oilhigh=10000; % upper limit for oil price values to be used in computations
mmax=floor(log(oilhigh)/log(alfa)); %corresponding index value

% specify finer grids to be used in computations
mgrid=[0:mmax]; %indexes for oil price process
kgrid=[0:100:40000]; % capacity (to get smoother figures, make a finer grid, e.g. kgrid=[0:1:40000]).
xgrid=(alfa.*ones(size(mgrid))).^mgrid; %oil price process

GRIDMIN=50; % this is the grid size to calculate value functions. Defines the lowest x-value for the grid.
TOLE=0.0001; %tolerance for value function convergence

Feed=0; %start with base case of no feed-in tariff
[Keqind0,Peqmax0]=ComputeEquilibrium(breakevenprice,Feed,mu,sigma,delta,r,mgrid,kgrid,xgrid,GRIDMIN,TOLE);

Feed=20;
[Keqind20,Peqmax20]=ComputeEquilibrium(breakevenprice,Feed,mu,sigma,delta,r,mgrid,kgrid,xgrid,GRIDMIN,TOLE);

Feed=30;
[Keqind30,Peqmax30]=ComputeEquilibrium(breakevenprice,Feed,mu,sigma,delta,r,mgrid,kgrid,xgrid,GRIDMIN,TOLE);

Feed=40;
[Keqind40,Peqmax40]=ComputeEquilibrium(breakevenprice,Feed,mu,sigma,delta,r,mgrid,kgrid,xgrid,GRIDMIN,TOLE);

figure;hold on;
plot(xgrid,Peqmax0);plot(xgrid,Peqmax20);plot(xgrid,Peqmax30);plot(xgrid,Peqmax40);
axis([100 650 50 350]);
title('Equilibrium price cost margin at tariff levels tau=0,20,30,40 with rI=50 and sigma=0.2. Corresponds to Figure 15 in the paper');


%Calculate welfare effects:
startind=153; %Starting point in xgrid for computing welfare effects
[ConsumerGain20,ProducerLoss20,WelfareLoss20]=ComputeWelfareEffects(Keqind0,Keqind20,20,xgrid,kgrid,startind,breakevenprice,delta,disc,q);
[ConsumerGain30,ProducerLoss30,WelfareLoss30]=ComputeWelfareEffects(Keqind0,Keqind30,30,xgrid,kgrid,startind,breakevenprice,delta,disc,q);
[ConsumerGain40,ProducerLoss40,WelfareLoss40]=ComputeWelfareEffects(Keqind0,Keqind40,40,xgrid,kgrid,startind,breakevenprice,delta,disc,q);

figure;hold on;
plot(20,ConsumerGain20,'+');plot(20,-ProducerLoss20,'*');plot(20,-WelfareLoss20,'o')
plot(30,ConsumerGain30,'+');plot(30,-ProducerLoss30,'*');plot(30,-WelfareLoss30,'o')
plot(40,ConsumerGain40,'+');plot(40,-ProducerLoss40,'*');plot(40,-WelfareLoss40,'o')
axis([0 50 -3e+10 3e+10]);
grid on;
title('Consumer gain (+), producer loss (*), and welfare loss (o) for tau=20,30,40. Corresponds to selected data points of Figure 16 in the paper');
